#!/usr/bin/env python3
"""
Exploit Detector
Advanced exploit and vulnerability detection tool.
Parameters:
    - target: Target system or application to analyze
    - mode: Analysis mode (binary, web, network)
    - output: Output file for results
Requires: capstone, requests, beautifulsoup4 packages
"""
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
from collections import defaultdict
import os
import re
import magic
import binascii
import struct
import socket
import requests
from bs4 import BeautifulSoup
from capstone import *
import yara

class ExploitDetector:
    def __init__(self, target, mode='binary'):
        self.target = target
        self.mode = mode.lower()
        self.results = defaultdict(dict)
        self.setup_logging()
        self.load_rules()
        self.initialize_patterns()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_rules(self):
        """Load YARA rules for exploit detection"""
        try:
            rules_text = """
            rule Shellcode_Patterns {
                strings:
                    $shell1 = { 31 c0 50 68 2f 2f 73 68 68 2f 62 69 6e }  // /bin/sh
                    $shell2 = { 31 c0 50 68 2f 62 61 73 68 }              // /bash
                    $shell3 = { b0 0b cd 80 }                             // execve
                    $shell4 = { ff d4 }                                   // call esp
                    $shell5 = { ff e4 }                                   // jmp esp
                    $nop1 = { 90 90 90 90 }                              // NOP sled
                    $nop2 = { 41 41 41 41 }                              // AAAA buffer
                condition:
                    any of them
            }

            rule Memory_Corruption {
                strings:
                    $fmt1 = "%n"                                         // Format string
                    $fmt2 = "%p"                                         // Address leak
                    $fmt3 = "%x"                                         // Memory dump
                    $heap1 = "free("
                    $heap2 = "malloc("
                    $heap3 = "realloc("
                    $overflow1 = "strcpy"
                    $overflow2 = "strcat"
                    $overflow3 = "gets"
                    $overflow4 = "sprintf"
                condition:
                    any of them
            }

            rule Web_Attacks {
                strings:
                    $sql1 = "UNION SELECT" nocase
                    $sql2 = "OR 1=1" nocase
                    $sql3 = "' OR '1'='1" nocase
                    $xss1 = "<script>" nocase
                    $xss2 = "javascript:" nocase
                    $xss3 = "onerror=" nocase
                    $cmd1 = "; ls" nocase
                    $cmd2 = "| cat" nocase
                    $cmd3 = "& dir" nocase
                condition:
                    any of them
            }
            """
            self.rules = yara.compile(source=rules_text)
        except Exception as e:
            self.logger.error(f"Error loading YARA rules: {str(e)}")
            self.rules = None

    def initialize_patterns(self):
        """Initialize exploit patterns and signatures"""
        self.patterns = {
            'rop_gadgets': [
                rb'\xc3',              # ret
                rb'\x5b\xc3',          # pop ebx; ret
                rb'\x5d\xc3',          # pop ebp; ret
                rb'\x5f\xc3',          # pop edi; ret
                rb'\x5e\xc3',          # pop esi; ret
                rb'\x58\xc3',          # pop eax; ret
                rb'\x59\xc3',          # pop ecx; ret
                rb'\x5a\xc3',          # pop edx; ret
            ],
            'web_attacks': {
                'sqli': [
                    r"'.*OR.*'1'.*='1",
                    r"admin'--",
                    r"UNION.*SELECT",
                    r"AND 1=1--",
                    r"OR 1=1--",
                ],
                'xss': [
                    r"<script.*>",
                    r"javascript:",
                    r"onerror=",
                    r"onload=",
                    r"eval\(",
                ],
                'cmdi': [
                    r";.*\w+",
                    r"\|.*\w+",
                    r"`.*\w+`",
                    r"\$\(.*\)",
                ],
                'path_traversal': [
                    r"\.\.\/",
                    r"\.\.\\",
                    r"%2e%2e%2f",
                    r"%252e%252e%252f",
                ],
            },
            'buffer_overflow': {
                'patterns': [
                    b"A" * 256,
                    b"A" * 512,
                    b"A" * 1024,
                    b"%x" * 64,
                    b"%n" * 16,
                ],
            }
        }

    def analyze_binary(self, binary_path):
        """Analyze binary for potential exploits"""
        try:
            with open(binary_path, 'rb') as f:
                binary_data = f.read()

            binary_info = {
                'path': binary_path,
                'size': len(binary_data),
                'type': magic.from_file(binary_path),
                'md5': hashlib.md5(binary_data).hexdigest(),
                'vulnerabilities': [],
                'rop_gadgets': [],
                'shellcode': [],
                'yara_matches': []
            }

            # YARA scanning
            if self.rules:
                matches = self.rules.match(data=binary_data)
                if matches:
                    binary_info['yara_matches'] = [match.rule for match in matches]

            # ROP gadget detection
            md = Cs(CS_ARCH_X86, CS_MODE_32)
            for pattern in self.patterns['rop_gadgets']:
                offset = 0
                while True:
                    offset = binary_data.find(pattern, offset)
                    if offset == -1:
                        break
                    
                    # Disassemble potential gadget
                    gadget_bytes = binary_data[offset:offset+16]
                    for insn in md.disasm(gadget_bytes, offset):
                        binary_info['rop_gadgets'].append({
                            'address': hex(offset),
                            'bytes': binascii.hexlify(pattern).decode(),
                            'disasm': f"{insn.mnemonic} {insn.op_str}"
                        })
                        break
                    offset += 1

            # Check for dangerous functions
            dangerous_functions = [
                b'strcpy', b'strcat', b'gets', b'sprintf',
                b'system', b'exec', b'popen'
            ]
            for func in dangerous_functions:
                if func in binary_data:
                    binary_info['vulnerabilities'].append({
                        'type': 'dangerous_function',
                        'function': func.decode(),
                        'risk': 'high'
                    })

            # Check for format string vulnerabilities
            format_strings = re.findall(b'%.?[sdixXnp]', binary_data)
            if format_strings:
                binary_info['vulnerabilities'].append({
                    'type': 'format_string',
                    'count': len(format_strings),
                    'risk': 'medium'
                })

            return binary_info

        except Exception as e:
            self.logger.error(f"Error analyzing binary {binary_path}: {str(e)}")
            return None

    def analyze_web_endpoint(self, url):
        """Analyze web endpoint for potential exploits"""
        try:
            endpoint_info = {
                'url': url,
                'vulnerabilities': [],
                'response_info': {},
                'security_headers': {}
            }

            # Send requests with different payloads
            session = requests.Session()
            response = session.get(
                url,
                verify=False,
                timeout=10,
                headers={'User-Agent': 'ExploitDetector/1.0'}
            )

            # Check security headers
            security_headers = {
                'X-Frame-Options': 'clickjacking',
                'X-XSS-Protection': 'xss',
                'X-Content-Type-Options': 'mime-sniffing',
                'Content-Security-Policy': 'csp',
                'Strict-Transport-Security': 'hsts'
            }

            for header, vuln_type in security_headers.items():
                if header not in response.headers:
                    endpoint_info['vulnerabilities'].append({
                        'type': f'missing_{vuln_type}_protection',
                        'risk': 'medium'
                    })

            # Store response headers
            endpoint_info['security_headers'] = dict(response.headers)

            # Test for SQL injection
            for pattern in self.patterns['web_attacks']['sqli']:
                test_url = f"{url}?id={pattern}"
                try:
                    response = session.get(test_url, timeout=5)
                    if any(error in response.text.lower() for error in [
                        'sql', 'mysql', 'oracle', 'syntax'
                    ]):
                        endpoint_info['vulnerabilities'].append({
                            'type': 'sql_injection',
                            'pattern': pattern,
                            'risk': 'high'
                        })
                except:
                    continue

            # Test for XSS
            for pattern in self.patterns['web_attacks']['xss']:
                test_url = f"{url}?q={pattern}"
                try:
                    response = session.get(test_url, timeout=5)
                    if pattern.encode() in response.content:
                        endpoint_info['vulnerabilities'].append({
                            'type': 'xss',
                            'pattern': pattern,
                            'risk': 'high'
                        })
                except:
                    continue

            # Test for command injection
            for pattern in self.patterns['web_attacks']['cmdi']:
                test_url = f"{url}?cmd={pattern}"
                try:
                    response = session.get(test_url, timeout=5)
                    if any(cmd in response.text.lower() for cmd in [
                        'root:', 'usr', 'win.ini', 'system32'
                    ]):
                        endpoint_info['vulnerabilities'].append({
                            'type': 'command_injection',
                            'pattern': pattern,
                            'risk': 'critical'
                        })
                except:
                    continue

            return endpoint_info

        except Exception as e:
            self.logger.error(f"Error analyzing endpoint {url}: {str(e)}")
            return None

    def analyze_network_service(self, host, port):
        """Analyze network service for potential exploits"""
        try:
            service_info = {
                'host': host,
                'port': port,
                'service': None,
                'banner': None,
                'vulnerabilities': []
            }

            # Connect to service
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.settimeout(5)
            sock.connect((host, port))

            # Try to get banner
            try:
                service_info['banner'] = sock.recv(1024).decode('utf-8', 'ignore')
            except:
                pass

            # Test for buffer overflow
            for pattern in self.patterns['buffer_overflow']['patterns']:
                try:
                    sock.send(pattern)
                    response = sock.recv(1024)
                    if not response or len(response) == 0:
                        service_info['vulnerabilities'].append({
                            'type': 'potential_buffer_overflow',
                            'pattern_length': len(pattern),
                            'risk': 'high'
                        })
                except:
                    continue

            # Test for format string
            format_string_test = b'%x' * 32
            try:
                sock.send(format_string_test)
                response = sock.recv(1024)
                if re.search(rb'[0-9a-f]{8}', response):
                    service_info['vulnerabilities'].append({
                        'type': 'format_string',
                        'risk': 'high'
                    })
            except:
                pass

            sock.close()
            return service_info

        except Exception as e:
            self.logger.error(
                f"Error analyzing service {host}:{port}: {str(e)}"
            )
            return None

    def save_results(self, output_file):
        """Save analysis results to file"""
        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'target': self.target,
            'mode': self.mode,
            'results': dict(self.results)
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        """Print analysis summary"""
        self.logger.info("\nExploit Detection Summary:")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Mode: {self.mode}")

        if 'binary' in self.results:
            binary = self.results['binary']
            self.logger.info("\nBinary Analysis:")
            self.logger.info(f"File: {binary.get('path')}")
            self.logger.info(f"Type: {binary.get('type')}")
            
            if binary.get('vulnerabilities'):
                self.logger.info("\nVulnerabilities Found:")
                for vuln in binary['vulnerabilities']:
                    self.logger.info(
                        f"- {vuln['type']} (Risk: {vuln['risk']})"
                    )
            
            if binary.get('rop_gadgets'):
                self.logger.info(
                    f"\nROP Gadgets Found: {len(binary['rop_gadgets'])}"
                )
                for gadget in binary['rop_gadgets'][:5]:  # Show first 5
                    self.logger.info(
                        f"- {gadget['address']}: {gadget['disasm']}"
                    )

        if 'web' in self.results:
            web = self.results['web']
            self.logger.info("\nWeb Analysis:")
            self.logger.info(f"URL: {web.get('url')}")
            
            if web.get('vulnerabilities'):
                self.logger.info("\nVulnerabilities Found:")
                for vuln in web['vulnerabilities']:
                    self.logger.info(
                        f"- {vuln['type']} (Risk: {vuln['risk']})"
                    )
            
            if web.get('security_headers'):
                self.logger.info("\nMissing Security Headers:")
                for header in web['security_headers']:
                    self.logger.info(f"- {header}")

        if 'network' in self.results:
            network = self.results['network']
            self.logger.info("\nNetwork Service Analysis:")
            self.logger.info(
                f"Service: {network.get('host')}:{network.get('port')}"
            )
            
            if network.get('vulnerabilities'):
                self.logger.info("\nVulnerabilities Found:")
                for vuln in network['vulnerabilities']:
                    self.logger.info(
                        f"- {vuln['type']} (Risk: {vuln['risk']})"
                    )

    def start_analysis(self, output_file=None):
        """Start exploit detection analysis"""
        self.logger.info(f"Starting exploit detection analysis")
        self.logger.info(f"Target: {self.target}")
        self.logger.info(f"Mode: {self.mode}")
        
        try:
            if self.mode == 'binary':
                result = self.analyze_binary(self.target)
                if result:
                    self.results['binary'] = result
                    
            elif self.mode == 'web':
                result = self.analyze_web_endpoint(self.target)
                if result:
                    self.results['web'] = result
                    
            elif self.mode == 'network':
                host, port = self.target.split(':')
                result = self.analyze_network_service(host, int(port))
                if result:
                    self.results['network'] = result
                
        except KeyboardInterrupt:
            self.logger.info("Analysis interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during analysis: {str(e)}")
        finally:
            if output_file:
                self.save_results(output_file)
            self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Exploit Detector")
    parser.add_argument("target",
                      help="Target system or application to analyze")
    parser.add_argument("--mode",
                      choices=['binary', 'web', 'network'],
                      default='binary',
                      help="Analysis mode")
    parser.add_argument("--output",
                      help="Output file for analysis results")
    args = parser.parse_args()

    detector = ExploitDetector(args.target, args.mode)
    detector.start_analysis(args.output)
